CREATE TABLE "TrafficTypes"(
	"TrafficType" NUMBER(10,0) NOT NULL,
	"Text" NVARCHAR2(100) NULL,
 CONSTRAINT "PK_TrafficTypes" PRIMARY KEY  
(
	"TrafficType"
)
) 
/
CREATE TABLE "PageEventDefinitions"(
	"PageEventDefinitionId" CHAR(32) NOT NULL,
	"Name" NVARCHAR2(100) NULL,
	"Value" NUMBER(10,0) NOT NULL,
	"IsSystem" "bit" NOT NULL,
	"IsAuthorFeedback" "bit" NOT NULL,
	"IsGoal" "bit" NOT NULL,
	"IsFailure" "bit" NOT NULL,
 CONSTRAINT "PK_PageEventDefinitions" PRIMARY KEY  
(
	"PageEventDefinitionId"
)
) 
/
CREATE  INDEX "IX_Goals" ON "PageEventDefinitions" 
(
	"IsGoal"
)
/
CREATE TABLE "OS"(
	"OsId" CHAR(32) NOT NULL,
	"MajorName" NVARCHAR2(250) NULL,
	"MinorName" NVARCHAR2(250) NULL,
	"Version" NVARCHAR2(250) NULL,
 CONSTRAINT "PK_OS" PRIMARY KEY  
(
	"OsId"
)
) 
/
CREATE TABLE "VisitorClassifications"(
	"VisitorClassification" NUMBER(10,0) NOT NULL,
	"Text" NVARCHAR2(100) NULL,
 CONSTRAINT "PK_VisitorClassification" PRIMARY KEY  
(
	"VisitorClassification"
)
) 
/
CREATE TABLE "Campaigns"(
	"CampaignId" CHAR(32) NOT NULL,
	"CampaignName" NVARCHAR2(100) NULL,
	"Cost" NUMBER(10,0) NOT NULL,
	"CostPerClick" NUMBER(10,0) NOT NULL,
	"CostPerDay" NUMBER(10,0) NOT NULL,
	"CostBase" NUMBER(10,0) NOT NULL,
	"Category1Label" NVARCHAR2(100) NULL,
	"Category1Id" CHAR(32) NULL,
	"Category2Label" NVARCHAR2(100) NULL,
	"Category2Id" CHAR(32) NULL,
	"Category3Label" NVARCHAR2(100) NULL,
	"Category3Id" CHAR(32) NULL,
	"IntegrationLabel" NVARCHAR2(100) NULL,
	"IntegrationId" CHAR(32) NULL,
	"IsActive" "bit" NOT NULL,
	"StartDate" TIMESTAMP(3) NOT NULL,
	"EndDate" TIMESTAMP(3) NOT NULL,
	"Data" NCLOB NULL,
 CONSTRAINT "PK_Campaign" PRIMARY KEY  
(
	"CampaignId"
)
) 
/
CREATE  INDEX "IX_Category1" ON "Campaigns" 
(
	"Category1Id",
	"CampaignId"
)
/
CREATE  INDEX "IX_Category2" ON "Campaigns" 
(
	"Category1Id",
	"Category2Id",
	"CampaignId"
)
/
CREATE  INDEX "IX_Category3" ON "Campaigns" 
(
	"Category1Id",
	"Category2Id",
	"Category3Id",
	"CampaignId"
)
/
CREATE TABLE "Browsers"(
	"BrowserId" CHAR(32) NOT NULL,
	"MajorName" NVARCHAR2(50) NULL,
	"MinorName" NVARCHAR2(50) NULL,
	"Version" NVARCHAR2(16) NULL,
 CONSTRAINT "PK_Browsers" PRIMARY KEY  
(
	"BrowserId"
)
) 
/
CREATE TABLE "Keywords"(
	"KeywordsId" CHAR(32) NOT NULL,
	"Text" NCLOB NULL,
 CONSTRAINT "PK_Keywords" PRIMARY KEY  
(
	"KeywordsId"
)
) 
/
CREATE TABLE "ItemUrls"(
	"ItemId" CHAR(32) NOT NULL,
	"Url" NVARCHAR2(400) NULL,
 CONSTRAINT "PK_ItemUrls" PRIMARY KEY  
(
	"ItemId"
)
) 
/
CREATE FUNCTION "GetVisitorClassification"
(
           @overridden int,
           @p0 int,
           @p1 int,
           @p2 int,
           @p3 int,
           @p4 int
)
RETURNS int
AS
BEGIN
  DECLARE @Return int

  IF @overridden = 0 OR @overridden = 925 /* Autodetects bot - special logic */
  BEGIN
    SELECT
    @Return = @p0

    SELECT
    @Return = CASE WHEN @Return IS NULL OR @p1 > @Return THEN @p1 ELSE @Return END
    SELECT
    @Return = CASE WHEN @Return IS NULL OR @p2 > @Return THEN @p2 ELSE @Return END
    SELECT
    @Return = CASE WHEN @Return IS NULL OR @p3 > @Return THEN @p3 ELSE @Return END
    SELECT
    @Return = CASE WHEN @Return IS NULL OR @p4 > @Return THEN @p4 ELSE @Return END

    if @overridden = 925 AND @Return < 900 /* If Autodetected as a robot (925), and previous classification is a human, still return 925 */
    BEGIN
      SELECT @Return = @overridden
    END

  END 
  ELSE BEGIN
    SELECT @Return = @overridden
  END        

  RETURN @Return
END
/
CREATE TABLE "TestDefinitions"(
	"TestSetId" CHAR(32) NOT NULL,
	"TestSetName" NVARCHAR2(100) NULL,
	"VariableId" CHAR(32) NOT NULL,
	"VariableLabel" NVARCHAR2(100) NULL,
	"VariableIndex" "smallint" NOT NULL,
	"ValueId" CHAR(32) NOT NULL,
	"ValueLabel" NVARCHAR2(100) NULL,
	"Value" "smallint" NOT NULL,
 CONSTRAINT "PK_TestDefinitions" PRIMARY KEY  
(
	"ValueId"
)
) 
/
CREATE TABLE "Status"(
	"StatusId" CHAR(32) NOT NULL,
	"InstanceName" NVARCHAR2(50) NULL,
	"Text" NVARCHAR2(100) NULL,
	"LastSuccessful" TIMESTAMP(3) NULL,
	"LastFailed" TIMESTAMP(3) NULL,
 CONSTRAINT "PK_Status" PRIMARY KEY  
(
	"StatusId"
)
) 
/
CREATE PROCEDURE "sp_sc_enable_indexes"
AS
  declare @tablename varchar(100), @indexname varchar(100), @sql nvarchar(1000)
  declare indexes cursor for select OBJECT_NAME(object_id) tablename, name indexname from sys.indexes where is_disabled = 1 order by type_desc
BEGIN
  open indexes
  fetch next from indexes into @tablename, @indexname
  while (@@FETCH_STATUS<>-1) begin
  set @sql = 'ALTER INDEX ' + @indexname + ' ON ' + @tablename + ' REBUILD'
  exec sp_executesql @sql
  fetch next from indexes into @tablename, @indexname
  end
  close indexes
  deallocate indexes
END
/
CREATE PROCEDURE "sp_sc_disable_indexes" 
AS
  declare @tablename varchar(100), @indexname varchar(100), @sql nvarchar(1000)
  declare indexes cursor for 
    select OBJECT_NAME(sys.indexes.object_id) tablename, sys.indexes.name indexname
    from 
      sys.indexes 
      inner join sys.views on sys.indexes.object_id = sys.views.object_id 
    where 
       sys.indexes.type_desc = ''
BEGIN
  open indexes
  fetch next from indexes into @tablename, @indexname
  while (@@FETCH_STATUS<>-1) begin
  set @sql = 'ALTER INDEX ' + @indexname + ' ON ' + @tablename + ' DISABLE'
  exec sp_executesql @sql
  fetch next from indexes into @tablename, @indexname
  end
  close indexes
  deallocate indexes
END
/
CREATE TABLE "Screens"(
	"ScreenId" CHAR(32) NOT NULL,
	"Text" NCLOB NULL,
	"Width" NUMBER(10,0) NOT NULL,
	"Height" NUMBER(10,0) NOT NULL,
 CONSTRAINT "PK_Screens" PRIMARY KEY  
(
	"ScreenId"
)
) 
/
CREATE TABLE "ReferringSites"(
	"ReferringSiteId" CHAR(32) NOT NULL,
	"Host" NVARCHAR2(100) NULL,
	"TrafficType" NUMBER(10,0) NOT NULL,
 CONSTRAINT "PK_ReferringSites" PRIMARY KEY  
(
	"ReferringSiteId"
)
) 
/
CREATE TABLE "GeoIps"(
	"Ip" "varbinary"(16) NOT NULL,
	"VisitorClassification" NUMBER(10,0) NOT NULL,
	"CachedValue" NCLOB NULL,
	"Fetched" TIMESTAMP(3) NULL,
 CONSTRAINT "PK_GeoIps" PRIMARY KEY  
(
	"Ip"
)
) 
/
CREATE TABLE "Locations"(
	"LocationId" CHAR(32) NOT NULL,
	"BusinessName" NVARCHAR2(100) NULL,
	"Country" NVARCHAR2(100) NULL,
	"VisitorClassification" NUMBER(10,0) NOT NULL,
	"IntegrationId" CHAR(32) NULL,
	"IntegrationLabel" NVARCHAR2(50) NULL,
	"ExternalUser" NVARCHAR2(100) NULL,
 CONSTRAINT "PK_Location" PRIMARY KEY  
(
	"LocationId"
)
) 
/
CREATE TABLE "Automations"(
	"AutomationId" CHAR(32) NOT NULL,
	"AutomationName" NVARCHAR2(100) NULL,
	"Campaignid" CHAR(32) NULL,
	"IntegrationId" CHAR(32) NULL,
	"IntegrationLabel" NVARCHAR2(100) NULL,
	"Suspended" "bit" NOT NULL,
 CONSTRAINT "PK_Automation" PRIMARY KEY  
(
	"AutomationId"
)
) 
/
CREATE TABLE "Visitors"(
	"VisitorId" CHAR(32) NOT NULL,
	"VisitorClassification" NUMBER(10,0) NOT NULL,
	"OverrideVisitorClassification" NUMBER(10,0) NOT NULL,
	"VisitCount" NUMBER(10,0) NOT NULL,
	"Value" NUMBER(10,0) NOT NULL,
	"AuthenticationLevel" NUMBER(10,0) NOT NULL,
	"ExternalUser" NVARCHAR2(100) NULL,
	"IntegrationId" CHAR(32) NULL,
	"IntegrationLabel" NVARCHAR2(100) NULL,
 CONSTRAINT "PK_Visitors" PRIMARY KEY  
(
	"VisitorId"
)
) 
/
CREATE TABLE "UserAgents"(
	"UserAgentId" CHAR(32) NOT NULL,
	"UserAgent" NCLOB NULL,
	"CachedValue" NCLOB NULL,
	"Fetched" TIMESTAMP(3) NULL,
	"VisitorClassification" NUMBER(10,0) NOT NULL,
 CONSTRAINT "PK_UserAgents" PRIMARY KEY  
(
	"UserAgentId"
)
) 
/
CREATE TABLE "NotificationSubscriptions"(
	"NotificationSubscriptionsId" CHAR(32) NOT NULL,
	"LocationId" CHAR(32) NOT NULL,
	"MailRecipients" NCLOB NULL,
 CONSTRAINT "PK_NotificationSubscriptions" PRIMARY KEY  
(
	"NotificationSubscriptionsId"
)
) 
/
CREATE TABLE "AutomationStates"(
	"AutomationStateId" CHAR(32) NOT NULL,
	"AutomationId" CHAR(32) NOT NULL,
	"VisitorId" CHAR(32) NULL,
	"UserName" NVARCHAR2(100) NULL,
	"CreatedDateTime" TIMESTAMP(3) NOT NULL,
	"EntryDateTime" TIMESTAMP(3) NOT NULL,
	"WakeupDateTime" TIMESTAMP(3) NULL,
	"StateId" CHAR(32) NULL,
	"StateName" NVARCHAR2(100) NULL,
	"TestSetId" CHAR(32) NULL,
	"TestValues" "binary"(16) NULL,
	"Data" NCLOB NULL,
	"State" NUMBER(10,0) NOT NULL,
	"DataCode" NUMBER(10,0) NOT NULL,
	"LastAccessedDateTime" TIMESTAMP(3) NULL,
 CONSTRAINT "PK_AutomationState" PRIMARY KEY  
(
	"AutomationStateId"
)
) 
/
CREATE  INDEX "IX_State" ON "AutomationStates" 
(
	"State",
	"WakeupDateTime"
)
/
CREATE TABLE "Visits"(
	"VisitId" CHAR(32) NOT NULL,
	"VisitorId" CHAR(32) NOT NULL,
	"VisitorVisitIndex" NUMBER(10,0) NOT NULL,
	"VisitPageCount" NUMBER(10,0) NOT NULL,
	"StartDateTime" TIMESTAMP(3) NOT NULL,
	"EndDateTime" TIMESTAMP(3) NOT NULL,
	"TrafficType" NUMBER(10,0) NOT NULL,
	"Value" NUMBER(10,0) NOT NULL,
	"AspNetSessionId" NVARCHAR2(50) NULL,
	"ReferringSiteId" CHAR(32) NOT NULL,
	"KeywordsId" CHAR(32) NOT NULL,
	"BrowserId" CHAR(32) NOT NULL,
	"UserAgentId" CHAR(32) NOT NULL,
	"OsId" CHAR(32) NOT NULL,
	"ScreenId" CHAR(32) NOT NULL,
	"CampaignId" CHAR(32) NULL,
	"RDNS" NVARCHAR2(450) NULL,
	"MultiSite" NVARCHAR2(450) NULL,
	"LocationId" CHAR(32) NOT NULL,
	"Ip" "varbinary"(16) NOT NULL,
	"BusinessName" NVARCHAR2(100) NULL,
	"City" NVARCHAR2(100) NULL,
	"PostalCode" NVARCHAR2(100) NULL,
	"MetroCode" NVARCHAR2(100) NULL,
	"AreaCode" NVARCHAR2(100) NULL,
	"Region" NVARCHAR2(100) NULL,
	"IspName" NVARCHAR2(100) NULL,
	"Country" NVARCHAR2(100) NULL,
	"Latitude" FLOAT NOT NULL,
	"Longitude" FLOAT NOT NULL,
	"TestSetId" CHAR(32) NULL,
	"TestValues" "binary"(16) NULL,
	"Referrer" NCLOB NULL,
 CONSTRAINT "PK_Visits" PRIMARY KEY  
(
	"VisitId"
)
) 
/
CREATE  INDEX "IX_ByDate" ON "Visits" 
(
	"StartDateTime",
	"VisitorId",
	"VisitId"
)
/
CREATE  INDEX "IX_ByLocation" ON "Visits" 
(
	"LocationId",
	"StartDateTime"
)
/
CREATE  INDEX "IX_ByReferringSite" ON "Visits" 
(
	"ReferringSiteId"
)
/
CREATE  INDEX "IX_ByVisitor" ON "Visits" 
(
	"VisitorId",
	"VisitId"
)
/
CREATE  INDEX "IX_FirstVisits" ON "Visits" 
(
	"VisitorVisitIndex",
	"VisitorId",
	"TrafficType"
)
/
CREATE TABLE "VisitorTags"(
	"VisitorTagId" CHAR(32) NOT NULL,
	"VisitorId" CHAR(32) NOT NULL,
	"TagName" NVARCHAR2(450) NULL,
	"TagValue" NCLOB NULL,
	"DateTime" TIMESTAMP(3) NOT NULL,
	"AcquaintanceId" CHAR(32) NOT NULL,
 CONSTRAINT "PK_VisitorTags" PRIMARY KEY  
(
	"VisitorTagId"
)
) 
/
CREATE  INDEX "IX_ByVisitor" ON "VisitorTags" 
(
	"VisitorId"
)
/
CREATE VIEW "ValueBySource" 
AS
SELECT CONVERT(nvarchar(8), Visits.StartDateTime, 112) Date, Visits.TrafficType, SUM(Visitors.VisitCount) Visits, SUM(Visitors.Value) Value, SUM(Visits.Value) FirstVisitValue, COUNT_BIG(*) Visitors
FROM dbo.Visits, dbo.Visitors
WHERE Visits.VisitorVisitIndex = 1 AND Visits.VisitorId = Visitors.VisitorId
GROUP BY CONVERT(nvarchar(8), Visits.StartDateTime, 112), TrafficType
/
CREATE UNIQUE  INDEX "PKey" ON "ValueBySource" 
(
	"Date",
	"TrafficType"
)
/
CREATE VIEW "VisitorsByLocation" 
AS
SELECT
    CONVERT(nvarchar(8), StartDateTime, 112) Date,
    Visitors.VisitorClassification,
	  LocationId,		
	  Visits.VisitorId,
	  COUNT_BIG(*) Visits,
	  SUM(Visits.Value) Value
FROM
	  dbo.Visits, dbo.Visitors
WHERE	
	  Visits.VisitorId = Visitors.VisitorId AND
	  Visitors.VisitorClassification < 900
GROUP BY
	  Visits.VisitorId,
		LocationId,
		VisitorClassification,
		CONVERT(nvarchar(8), StartDateTime, 112)
/
CREATE UNIQUE  INDEX "PK_VisitorsByLocation" ON "VisitorsByLocation" 
(
	"Date",
	"VisitorClassification",
	"LocationId",
	"VisitorId"
)
/
CREATE TABLE "Profiles"(
	"ProfileId" CHAR(32) NOT NULL,
	"VisitId" CHAR(32) NOT NULL,
	"VisitorId" CHAR(32) NOT NULL,
	"ProfileName" NVARCHAR2(50) NULL,
	"PatternValues" NCLOB NULL,
	"PatternLabel" NCLOB NULL,
	"PatternId" CHAR(32) NULL,
 CONSTRAINT "PK_Profile" PRIMARY KEY  
(
	"ProfileId"
)
) 
/
CREATE  INDEX "IX_ByVisitId" ON "Profiles" 
(
	"VisitId"
)
/
CREATE  INDEX "IX_ByVisitor" ON "Profiles" 
(
	"VisitorId"
)
/
CREATE TABLE "Pages"(
	"PageId" CHAR(32) NOT NULL,
	"VisitId" CHAR(32) NOT NULL,
	"VisitorId" CHAR(32) NOT NULL,
	"VisitPageIndex" NUMBER(10,0) NOT NULL,
	"DateTime" TIMESTAMP(3) NOT NULL,
	"ItemId" CHAR(32) NOT NULL,
	"ItemLanguage" NVARCHAR2(50) NULL,
	"ItemVersion" NUMBER(10,0) NOT NULL,
	"DeviceId" CHAR(32) NOT NULL,
	"Url" NVARCHAR2(800) NULL,
	"UrlText" NCLOB NULL,
	"TestSetId" CHAR(32) NULL,
	"TestValues" "binary"(16) NULL,
	"Duration" NUMBER(10,0) NULL,
	"Data" NCLOB NULL,
 CONSTRAINT "PK_Pages" PRIMARY KEY  
(
	"PageId"
)
) 
/
CREATE  INDEX "IX_ByDate" ON "Pages" 
(
	"DateTime"
)
/
CREATE  INDEX "IX_ByVisitor" ON "Pages" 
(
	"VisitorId",
	"VisitId"
)
/
CREATE TABLE "PageEvents"(
	"PageEventId" CHAR(32) NOT NULL,
	"VisitorId" CHAR(32) NOT NULL,
	"PageEventDefinitionId" CHAR(32) NOT NULL,
	"PageId" CHAR(32) NOT NULL,
	"VisitId" CHAR(32) NOT NULL,
	"ItemId" CHAR(32) NOT NULL,
	"IntegrationId" CHAR(32) NULL,
	"IntegrationLabel" NVARCHAR2(100) NULL,
	"DataKey" NVARCHAR2(100) NULL,
	"Data" NVARCHAR2(450) NULL,
	"DataCode" NUMBER(10,0) NOT NULL,
	"Text" NCLOB NULL,
	"Timestamp" INTEGER NOT NULL,
	"CustomSorting" NUMBER(10,0) NOT NULL,
	"DateTime" TIMESTAMP(3) NOT NULL,
 CONSTRAINT "PK_PageEvents" PRIMARY KEY  
(
	"PageEventId"
)
) 
/
CREATE  INDEX "IX_ByDate" ON "PageEvents" 
(
	"DateTime"
)
/
CREATE  INDEX "IX_ByDefinition" ON "PageEvents" 
(
	"PageEventDefinitionId",
	"VisitId",
	"DataKey",
	"ItemId"
)
/
CREATE  INDEX "IX_ByVisitor" ON "PageEvents" 
(
	"VisitorId",
	"VisitId"
)
/
CREATE TRIGGER "UpdateItemUrls" ON "Pages" 
FOR INSERT NOT FOR REPLICATION AS
BEGIN
  INSERT INTO ItemUrls (ItemId, Url) SELECT DISTINCT INSERTED.ItemId, INSERTED.Url FROM INSERTED LEFT JOIN ItemUrls ON INSERTED.ItemId = ItemUrls.ItemId WHERE ItemUrls.ItemId IS NULL
END
/
CREATE VIEW "TrafficByDay" 
SELECT
  CONVERT(nvarchar(8), Visits.StartDateTime, 112) AS Date, 
  CONVERT(nvarchar(6), Visits.StartDateTime, 112) + '01' Month,
  Visits.TrafficType, 
  Visits.CampaignId, 
  Pages.ItemId, 
  SUBSTRING(CONVERT(varchar(800), ItemUrls.Url), 1, CHARINDEX('?', ItemUrls.Url + '?')) Url, 
  Visits.KeywordsId, 
  CONVERT(nvarchar(400), Keywords.Text) Keywords, 
  Visits.ReferringSiteId, 
  ReferringSites.Host ReferringSite,
  (CASE WHEN Visits.VisitorVisitIndex = 1 THEN 1 ELSE 0 END) FirstVisit,
  COUNT_BIG(*) AS Visits, SUM(Visits.Value) AS Value
FROM 
  dbo.Visits
  INNER JOIN dbo.Keywords ON Visits.KeywordsId = Keywords.KeywordsId 
  INNER JOIN dbo.Pages ON Visits.VisitId = Pages.VisitId AND Pages.VisitPageIndex = 1 AND Pages.ItemId <> '{00000000-0000-0000-0000-000000000000}' 
  INNER JOIN dbo.ItemUrls ON Pages.ItemId = ItemUrls.ItemId 
  INNER JOIN dbo.ReferringSites ON Visits.ReferringSiteId = ReferringSites.ReferringSiteId
GROUP BY 
  CONVERT(nvarchar(8), Visits.StartDateTime, 112), CONVERT(nvarchar(6), Visits.StartDateTime, 112) + '01',
  Visits.TrafficType, Visits.CampaignId, Pages.ItemId, SUBSTRING(CONVERT(varchar(800), ItemUrls.Url), 1, CHARINDEX('?', ItemUrls.Url + '?')), 
  Visits.KeywordsId, CONVERT(nvarchar(400), Keywords.Text), 
  Visits.ReferringSiteId, ReferringSites.Host, (CASE WHEN Visits.VisitorVisitIndex = 1 THEN 1 ELSE 0 END)
/
CREATE UNIQUE  INDEX "PKey" ON "TrafficByDay" 
(
	"Date",
	"TrafficType",
	"KeywordsId",
	"ReferringSiteId",
	"CampaignId",
	"ItemId",
	"FirstVisit"
)
/
CREATE  INDEX "IX_ByCampaign" ON "TrafficByDay" 
(
	"CampaignId",
	"Month",
	"Date"
)
/
CREATE  INDEX "IX_ByType" ON "TrafficByDay" 
(
	"TrafficType",
	"Month",
	"Date"
)
/
CREATE VIEW "VisitEvents" 
AS
SELECT
  CONVERT(nvarchar(8), Visits.StartDateTime, 112) AS Date, CONVERT(nvarchar(6), Visits.StartDateTime, 112) + '01' AS Month,
  Visits.TrafficType, Visits.CampaignId, Visits.Value,
  PageEvents.VisitId, PageEvents.PageEventDefinitionId, PageEventDefinitions.Name PageEventName, 
  PageEventDefinitions.IsGoal, Pages.ItemId, PageEvents.DataKey,
  COUNT_BIG(*) NumberOfEvents
FROM dbo.PageEvents, dbo.Visits, dbo.PageEventDefinitions, dbo.Pages
WHERE 
  Visits.VisitId = PageEvents.VisitId AND 
  PageEvents.PageEventDefinitionId = PageEventDefinitions.PageEventDefinitionId AND
  PageEvents.PageId = Pages.PageId AND
  (PageEvents.DataKey <> '' OR PageEventDefinitions.IsGoal = 1)
GROUP BY 
  CONVERT(nvarchar(8), Visits.StartDateTime, 112), CONVERT(nvarchar(6), Visits.StartDateTime, 112) + '01',
  Visits.TrafficType, Visits.CampaignId, Visits.Value, 
  PageEvents.VisitId, PageEvents.PageEventDefinitionId, PageEventDefinitions.Name, PageEventDefinitions.IsGoal,
  PageEvents.DataKey, Pages.ItemId
/
CREATE UNIQUE  INDEX "PKey" ON "VisitEvents" 
(
	"IsGoal",
	"VisitId",
	"PageEventDefinitionId",
	"DataKey",
	"ItemId"
)
/
CREATE  INDEX "IX_EventsByCampaign" ON "VisitEvents" 
(
	"IsGoal",
	"PageEventDefinitionId",
	"CampaignId",
	"Month",
	"Date",
	"DataKey"
)
/
CREATE  INDEX "IX_GoalsByCampaign" ON "VisitEvents" 
(
	"IsGoal",
	"CampaignId",
	"PageEventDefinitionId",
	"Month",
	"Date",
	"DataKey"
)
/
CREATE  INDEX "IX_GoalsByItemId" ON "VisitEvents" 
(
	"ItemId",
	"IsGoal",
	"PageEventDefinitionId",
	"Date",
	"DataKey"
)
/
CREATE VIEW "TrafficOverview" AS
SELECT "Date"
      ,"Month"
      ,"TrafficType"
      ,"CampaignId"
      ,"ItemId"
      ,"Url"
      ,"KeywordsId"
      ,"Keywords"
      ,"ReferringSiteId"
      ,"ReferringSite"
      ,"FirstVisit"
      ,"Visits"
      ,"Value"
  FROM "TrafficByDay" 
/
CREATE VIEW "VisitEventsByPage"
AS
SELECT 
  Date, 
  Month,
  TrafficType,
  CampaignId,
  VisitId,
  PageEventDefinitionId,
  PageEventName,
  ItemId,
  Value,
  IsGoal,
  SUM(NumberOfEvents) NumberOfEvents
FROM VisitEvents WITH(NOEXPAND)
GROUP BY
  Date, 
  Month,
  TrafficType,
  CampaignId,
  VisitId,
  PageEventDefinitionId,
  PageEventName,
  ItemId,
  Value,
  IsGoal
/
CREATE VIEW "VisitEventsByDefinition"
AS
SELECT 
  Date, 
  Month,
  TrafficType,
  CampaignId,
  VisitId,
  PageEventDefinitionId,
  PageEventName,
  Value,
  IsGoal,
  SUM(NumberOfEvents) NumberOfEvents
FROM VisitEvents WITH(NOEXPAND)
GROUP BY
  Date, 
  Month,
  TrafficType,
  CampaignId,
  VisitId,
  PageEventDefinitionId,
  PageEventName,
  Value,
  IsGoal
/
CREATE VIEW "VisitEventsByDataKey"
AS
SELECT 
  Date, 
  Month,
  TrafficType,
  CampaignId,
  VisitId,
  PageEventDefinitionId,
  PageEventName,
  DataKey,
  Value,
  IsGoal,
  SUM(NumberOfEvents) NumberOfEvents
FROM VisitEvents WITH(NOEXPAND)
GROUP BY
  Date, 
  Month,
  TrafficType,
  CampaignId,
  VisitId,
  PageEventDefinitionId,
  PageEventName,
  DataKey,
  Value,
  IsGoal
/
CREATE VIEW "CampaignsOverview"
AS
SELECT 
  TrafficOverview.Date, TrafficOverview.Month,
  TrafficOverview.ItemId, TrafficOverview.Url, 
  TrafficOverview.KeywordsId, TrafficOverview.Keywords, TrafficOverview.ReferringSiteId, TrafficOverview.ReferringSite, 
  Campaigns.CampaignName, Campaigns.Category1Id, Campaigns.Category2Id, Campaigns.Category3Id,
  TrafficOverview.Visits, TrafficOverview.Value
FROM TrafficOverview, Campaigns 
WHERE TrafficOverview.CampaignId = Campaigns.CampaignId
/
ALTER TABLE "ReferringSites"  
ADD CONSTRAINT "TrafficType1" FOREIGN KEY ("TrafficType") REFERENCES "TrafficTypes" ("TrafficType")
/
ALTER TABLE "GeoIps"  
ADD CONSTRAINT "VisitorClassification10" FOREIGN KEY ("VisitorClassification") REFERENCES "VisitorClassifications" ("VisitorClassification")
/
ALTER TABLE "Locations"  
ADD CONSTRAINT "VisitorClassification2" FOREIGN KEY ("VisitorClassification") REFERENCES "VisitorClassifications" ("VisitorClassification")
/
ALTER TABLE "Automations"  
ADD CONSTRAINT "CampaignId20" FOREIGN KEY ("CampaignId") REFERENCES "Campaigns" ("CampaignId")
/
ALTER TABLE "Visitors"  
ADD CONSTRAINT "VisitorClassification21" FOREIGN KEY ("VisitorClassification") REFERENCES "VisitorClassifications" ("VisitorClassification")
/
ALTER TABLE "Visitors"  
ADD CONSTRAINT "VisitorClassification210" FOREIGN KEY ("VisitorClassification") REFERENCES "VisitorClassifications" ("VisitorClassification")
/
ALTER TABLE "UserAgents"  
ADD CONSTRAINT "VisitorClassification3" FOREIGN KEY ("VisitorClassification") REFERENCES "VisitorClassifications" ("VisitorClassification")
/
ALTER TABLE "NotificationSubscriptions"  
ADD CONSTRAINT "LocationId30" FOREIGN KEY ("LocationId") REFERENCES "Locations" ("LocationId")
/
ALTER TABLE "AutomationStates"  
ADD CONSTRAINT "VisitorId31" FOREIGN KEY ("VisitorId") REFERENCES "Visitors" ("VisitorId")
/
ALTER TABLE "AutomationStates"  
ADD CONSTRAINT "AutomationId310" FOREIGN KEY ("AutomationId") REFERENCES "Automations" ("AutomationId")
/
ALTER TABLE "Visits"  
ADD CONSTRAINT "BrowserId32" FOREIGN KEY ("BrowserId") REFERENCES "Browsers" ("BrowserId")
/
ALTER TABLE "Visits"  
ADD CONSTRAINT "CampaignId320" FOREIGN KEY ("CampaignId") REFERENCES "Campaigns" ("CampaignId")
/
ALTER TABLE "Visits"  
ADD CONSTRAINT "Ip321" FOREIGN KEY ("Ip") REFERENCES "GeoIps" ("Ip")
/
ALTER TABLE "Visits"  
ADD CONSTRAINT "KeywordsId3210" FOREIGN KEY ("KeywordsId") REFERENCES "Keywords" ("KeywordsId")
/
ALTER TABLE "Visits"  
ADD CONSTRAINT "LocationId33" FOREIGN KEY ("LocationId") REFERENCES "Locations" ("LocationId")
/
ALTER TABLE "Visits"  
ADD CONSTRAINT "OsId330" FOREIGN KEY ("OsId") REFERENCES "OS" ("OsId")
/
ALTER TABLE "Visits"  
ADD CONSTRAINT "ReferringSiteId331" FOREIGN KEY ("ReferringSiteId") REFERENCES "ReferringSites" ("ReferringSiteId")
/
ALTER TABLE "Visits"  
ADD CONSTRAINT "ScreenId3310" FOREIGN KEY ("ScreenId") REFERENCES "Screens" ("ScreenId")
/
ALTER TABLE "Visits"  
ADD CONSTRAINT "TrafficType332" FOREIGN KEY ("TrafficType") REFERENCES "TrafficTypes" ("TrafficType")
/
ALTER TABLE "Visits"  
ADD CONSTRAINT "UserAgentId3320" FOREIGN KEY ("UserAgentId") REFERENCES "UserAgents" ("UserAgentId")
/
ALTER TABLE "Visits"  
ADD CONSTRAINT "VisitorId3321" FOREIGN KEY ("VisitorId") REFERENCES "Visitors" ("VisitorId")
/
ALTER TABLE "VisitorTags"  
ADD CONSTRAINT "VisitorId33210" FOREIGN KEY ("VisitorId") REFERENCES "Visitors" ("VisitorId")
/
ALTER TABLE "Profiles"  
ADD CONSTRAINT "VisitId333" FOREIGN KEY ("VisitId") REFERENCES "Visits" ("VisitId")
/
ALTER TABLE "Profiles"  
ADD CONSTRAINT "VisitorId3330" FOREIGN KEY ("VisitorId") REFERENCES "Visitors" ("VisitorId")
/
ALTER TABLE "Pages"  
ADD CONSTRAINT "VisitorId3331" FOREIGN KEY ("VisitorId") REFERENCES "Visitors" ("VisitorId")
/
ALTER TABLE "Pages"  
ADD CONSTRAINT "VisitId33310" FOREIGN KEY ("VisitId") REFERENCES "Visits" ("VisitId")
/
ALTER TABLE "PageEvents"  
ADD CONSTRAINT "PageEventDefinitionId3332" FOREIGN KEY ("PageEventDefinitionId") REFERENCES "PageEventDefinitions" ("PageEventDefinitionId")
/
ALTER TABLE "PageEvents"  
ADD CONSTRAINT "PageId33320" FOREIGN KEY ("PageId") REFERENCES "Pages" ("PageId")
/
ALTER TABLE "PageEvents"  
ADD CONSTRAINT "VisitorId33321" FOREIGN KEY ("VisitorId") REFERENCES "Visitors" ("VisitorId")
/
ALTER TABLE "PageEvents"  
ADD CONSTRAINT "VisitId333210" FOREIGN KEY ("VisitId") REFERENCES "Visits" ("VisitId")
/

CREATE OR REPLACE FUNCTION ProfileKeyValue
(
  ProfileKeyDefinitionId IN "ProfileKeyDefinition"."ProfileKeyDefinitionId"%TYPE, 
  SessionId IN "Session"."SessionId"%TYPE
) 
RETURN FLOAT IS
  Value "ProfileKey"."Value"%TYPE;
  ProfileCount "Profile"."Count"%TYPE;
  Total "Profile"."Total"%TYPE;
  ProfileType "ProfileDefinition"."ProfileType"%TYPE;
BEGIN
  BEGIN
    SELECT
      "ProfileKey"."Value", 
      "Profile"."Count", 
      "Profile"."Total", 
      "ProfileDefinition"."ProfileType"
    INTO Value, ProfileCount, Total, ProfileType
    FROM 
      "ProfileKey", 
      "Profile", 
      "ProfileDefinition"
    WHERE
      rownum <= 1 and
      ProfileKeyDefinitionId = "ProfileKeyDefinitionId" and
      "ProfileKey"."ProfileId" = "Profile"."ProfileId" and
      "Profile"."ProfileDefinitionId" = "ProfileDefinition"."ProfileDefinitionId" and
      "Profile"."SessionId" = "SessionId";

    IF LOWER(TO_CHAR(ProfileType)) = 'sum' THEN
      return Value;
    ELSIF LOWER(TO_CHAR(ProfileType)) = 'percentage' THEN
      return Value / ProfileCount * 100;
    ELSIF LOWER(TO_CHAR(ProfileType)) = 'average' THEN
      return Value / Total;
    ELSE
      return 0;
    END IF;
  EXCEPTION
    WHEN NO_DATA_FOUND THEN
      Value := 0;
      ProfileCount:= 0;
      Total := 0;
      ProfileType := 'nothing';

      return 0;
  WHEN OTHERS THEN
    RAISE;
  END;
END ProfileKeyValue;
/
CREATE OR REPLACE FUNCTION GetVisitorIdentification
(
  overridden IN INTEGER,
  p0 IN INTEGER,
  p1 IN INTEGER,
  p2 IN INTEGER,
  p3 IN INTEGER,
  p4 IN INTEGER
) 
RETURN INTEGER
IS
  IsAutoDetected INTEGER;
  ReturnValue INTEGER;
  overriddenVariable INTEGER;
BEGIN
  IsAutoDetected := 0;
  ReturnValue := 0;
  overriddenVariable := overridden;

  IF overridden = 925 THEN
    IsAutoDetected := 1;
    overriddenVariable := 0;
  END IF;

  IF overriddenVariable = 0 THEN
    IF p0 > p1 AND p0 > p2 AND p0 > p3 AND p0 > p4 THEN
      ReturnValue := p0;
    ELSIF p1 > p2 and p1 > p3 and p1 > p4 THEN
      ReturnValue := p1;
    ELSIF p2 > p3 AND p2 > p4 THEN
      ReturnValue := p2;
    ELSIF p3 > p4 THEN
      ReturnValue := p3;
    ELSE
      ReturnValue := p4;
    END IF;
  ELSE
    ReturnValue := overriddenVariable;
  END IF;

  IF IsAutoDetected = 1 AND ReturnValue < 900 THEN
    ReturnValue := 925;
  END IF;

  return ReturnValue;
END GetVisitorIdentification;
/
exit